clear
close all
clc

rand('seed',3000);
randn('seed',3000);

%% Data Generation

% picks = double(1-logical(double(imread('.\Images\text256.png'))/255));
% picks = double(1-logical(double(imread('.\Images\text512.png'))/255));
picks = double(1-logical(double(imread('.\Images\mask512.png'))/255));
percentage = length(find(picks==0))/numel(picks);

img = double(rgb2gray(imread('.\Images\AngryBirds.jpg'))); img = img(:,276:787);

imname = 'AngryBirds.jpg';

[M,N] = size(img);

sigma = 4;
B = picks.*img;
B = B + sigma*randn(M,N);
% B(picks==0) = 0;
% B(picks==0) = 255*randn(size(find(picks==0)));
B(picks==0) = 255;
B = min(max(B,0),255);
figure; imshow(img,[0 255])
figure; imshow(img(91:230,201:340),[0 255]);
figure; imshow(B,[0 255]); title(['PSNR = ' num2str(psnr(B/255,img/255))]);
figure; imshow(B(91:230,201:340),[0 255]); title(['PSNR = ' num2str(psnr(B/255,img/255))]);

%% Reconstruction

frame1 = 3;
frame2 = 1;
Level = 1;
wLevel1 = 1;
wLevel2 = 1;

frame3 = 1;
Level_v = 4;
wLevel_v = -1;

[D1,~] = GenerateFrameletFilter(frame1);
[D2,~] = GenerateFrameletFilter(frame2);

W1 = @(x) FraDecMultiLevel(x,D1,Level);
W2 = @(x) FraDecMultiLevel(x,D2,Level);

nD1 = length(D1);
nD2 = length(D2);

lambda = 3.75;
gamma = 3;
rho1 = 0.2;
tau = 1;

tol = 1e-4;

muLevel1 = getwThresh(lambda,wLevel1,Level,D1);
muLevel2 = getwThresh(gamma,wLevel2,Level,D2);

[Gamma,GammaC] = UpdateSupportSplit(B,frame1,Level,1,tau); % Initialize Support

maxiter = 100;
ut = zeros(size(B));
% ut = B;
beta = 5;
ratio = 1;
ratio2 = 0.5;
opts5 = struct('mit_out',1000,'beta1',0.05,'beta2',0.05);
for ki = 1 : Level
    opts6{ki} = struct('beta',beta*ratio^(ki-1));
    rho{ki} = rho1*ratio2^(ki-1);
end

for Iter = 1 : maxiter
    u6 = WaveletFrameInpaintSD3(B,picks,frame1,frame2,Level,wLevel1,wLevel2,lambda,gamma,Gamma,GammaC,img,opts5);
    C1 = W1(u6); C2 = W2(u6);
    for ki = 1 : Level
        vec1 = zeros(size(B)); vec2 = zeros(size(B));
        for ji = 1 : nD1-1
            for jj = 1 : nD1-1
                vec1 = vec1 + muLevel1{ki}{ji,jj}.*C1{ki}{ji,jj}.^2;
            end
        end
        vec1 = sqrt(vec1);
        for ji = 1 : nD2-1
            for jj = 1 : nD2-1
                vec2 = vec2 + muLevel2{ki}{ji,jj}.*C2{ki}{ji,jj}.^2;
            end
        end
        vec2 = sqrt(vec2);
        [Gamma{ki},GammaC{ki}] = SplitBregSupportUpdate(vec1,vec2,B,frame3,Level_v,wLevel_v,rho{ki},opts6{ki});
    end
    
    err = norm(u6-ut,'fro')/norm(B,'fro'); ut = u6;
    display('======================');
    display(['PSNR at step ' num2str(Iter) ' is: ' num2str(psnr(u6/255,img/255)) '; Error = ' num2str(err)]);
    display('======================');
    if err < tol
        break
    end
end
PSNR = psnr(u6/255,img/255);
figure; imshow(u6,[0 255]); title(['PSNR = ' num2str(PSNR)]);
figure; imshow(u6(91:230,201:340),[0 255]); title(['PSNR = ' num2str(PSNR)]);
Edge = zeros([size(img),Level]);
for ki = 1 : Level
    Edge(:,:,ki) = double(Gamma{ki}>0.5);
end
figure; contour_superpose(u6,Edge);
figure; contour_superpose(u6(91:230,201:340),Edge(91:230,201:340,:));
clear D1 D2 nD1 nD2 W1 W2 ki ji jj C1 C2 vec1 vec2