function u = SplitBregDebSD3(g,ker,frame1,frame2,Level,wLevel1,wLevel2,lambda,gamma,Gamma,GammaC,oimg,opts)

if ~exist('opts','var'); opts = []; end

[D1,R1] = GenerateFrameletFilter(frame1);
W1 = @(x) FraDecMultiLevel(x,D1,Level);
W1T = @(x) FraRecMultiLevel(x,R1,Level);
nD1 = length(D1);

[D2,R2] = GenerateFrameletFilter(frame2);
W2 = @(x) FraDecMultiLevel(x,D2,Level);
W2T = @(x) FraRecMultiLevel(x,R2,Level);
nD2 = length(D2);

[m,n] = size(g);

F = @(x) fft2(x);
FT = @(x) ifft2(x);

% Set up the option fields
[mit_inn,mit_out,tol,beta1,beta2,u_ini] = setopts(opts);

% Compute the weighted thresholding parameters.
muLevel1 = getwThresh(lambda/beta1,wLevel1,Level,D1);
muLevel2 = getwThresh(gamma/beta2,wLevel2,Level,D2);

% Setting up Denominator Constants
eigKer = psf2otf(ker,[m n]);
Denom = abs(eigKer).^2 + beta1 + beta2;

Atg = conj(eigKer).*F(g);

normg = norm(g,'fro');

% Initialization
if isempty(u_ini); u = zeros(m,n); else u = u_ini; clear u_ini; end
d1 = W1(u); b1 = d1;
d2 = W2(u); b2 = d2;

%% Main Loop

for nstep = 1 : mit_out
    
    muLevel11 = CoeffOper('*',muLevel1,ReArrangeCell(GammaC,D1));
    muLevel12 = CoeffOper('*',muLevel2,ReArrangeCell(Gamma,D2));
    
    for innerstep = 1 : mit_inn
        
        % Solve u.
        C1 = CoeffOper('-',d1,b1);
        C2 = CoeffOper('-',d2,b2);
        u = FT((F(beta1*W1T(C1)+beta2*W2T(C2))+Atg)./Denom);
        u = min(max(u,0),255);
        
        % Solve d1
        C1 = W1(u);
        C1pb1 = CoeffOper('+',C1,b1);
        d1 = CoeffOper('vs',C1pb1,muLevel11);
        
        % Solve d2
        C2 = W2(u);
        C2pb2 = CoeffOper('+',C2,b2);
        d2 = CoeffOper('vs',C2pb2,muLevel12);
        
    end
    
    err = 0;
    
    for ki = 1 : Level
        for ji = 1 : nD1-1
            for jj = 1 : nD1-1
                if ((ji~=1)||(jj~=1))||(ki==Level)
                    deltab1 = C1{ki}{ji,jj} - d1{ki}{ji,jj};
                    err = err + norm(deltab1,'fro')^2;
                    b1{ki}{ji,jj} = b1{ki}{ji,jj} + deltab1;
                end
            end
        end
    end
    
    for ki = 1 : Level
        for ji = 1 : nD2-1
            for jj = 1 : nD2-1
                if ((ji~=1)||(jj~=1))||(ki==Level)
                    deltab2 = C2{ki}{ji,jj} - d2{ki}{ji,jj};
                    err = err + norm(deltab2,'fro')^2;
                    b2{ki}{ji,jj} = b2{ki}{ji,jj} + deltab2;
                end
            end
        end
    end
    
    err = sqrt(err)/normg;
    
    stopc = (err <= tol);

    if stopc == 1
        disp(['Problem Step = ' num2str(nstep) '; PSNR = ' num2str(psnr(u/255,oimg/255)) '; error = ' num2str(err)]);
        break;
    end
end
if stopc ~= 1
    disp(['Problem Step = ' num2str(nstep) '; PSNR = ' num2str(psnr(u/255,oimg/255)) '; error = ' num2str(err)]);
end

%% Subfunctions
function [mit_inn,mit_out,tol,beta1,beta2,u_ini]=setopts(opts)

% define default option fields
mit_inn = 1;
mit_out = 200;
tol = 1.e-4;
beta1 = 0.05;
beta2 = 0.05;
u_ini = [];

% change to specified option fields if exist
if ~isempty(opts);
    if ~isa(opts,'struct'); error('New_MSModel : opts not a struct'); end
    if isfield(opts,'mit_inn'); mit_inn = opts.mit_inn; end
    if isfield(opts,'mit_out'); mit_out = opts.mit_out; end
    if isfield(opts,'tol'); tol = opts.tol; end
    if isfield(opts,'beta1'); beta1 = opts.beta1; end
    if isfield(opts,'beta2'); beta2 = opts.beta2; end
    if isfield(opts,'u_ini'); u_ini = opts.u_ini; end
end
return;

function [D,R] = GenerateFrameletFilter(frame)

if (nargin < 1) || (isempty(frame)); frame = 1; end
if (frame~=0) && (frame~=1) && (frame~=3)
    error('Input variable frame must be 0, 1, or 3');
end
if frame==0          %Haar Wavelet
    D{1}=[0 1 1]/2;
    D{2}=[0 1 -1]/2;
%     D{3}='rr';
    D{3}='cc';
    R{1}=[1 1 0]/2;
    R{2}=[-1 1 0]/2;
%     R{3}='rr';
    R{3}='cc';
elseif frame==1      %Piecewise Linear Framelet
    D{1}=[1 2 1]/4;
    D{2}=[1 0 -1]/4*sqrt(2);
    D{3}=[-1 2 -1]/4;
%     D{4}='rrr';
    D{4}='ccc';
    R{1}=[1 2 1]/4;
    R{2}=[-1 0 1]/4*sqrt(2);
    R{3}=[-1 2 -1]/4;
%     R{4}='rrr';
    R{4}='ccc';
elseif frame==3      %Piecewise Cubic Framelet
    D{1}=[1 4 6 4 1]/16;
    D{2}=[1 2 0 -2 -1]/8;
    D{3}=[-1 0 2 0 -1]/16*sqrt(6);
    D{4}=[-1 2 0 -2 1]/8;
    D{5}=[1 -4 6 -4 1]/16;
    D{6}='ccccc';
%     D{6}='rrrrr';
    R{1}=[1 4 6 4 1]/16;
    R{2}=[-1 -2 0 2 1]/8;
    R{3}=[-1 0 2 0 -1]/16*sqrt(6);
    R{4}=[1 -2 0 2 -1]/8;
    R{5}=[1 -4 6 -4 1]/16;
    R{6}='ccccc';
%     R{6}='rrrrr';
end
return;