function [Gamma,GammaC] = SplitBregSupportUpdate(f1,f2,g,frame,Level,wLevel,rho,opts)

if ~exist('opts','var'); opts = []; end

[D,R] = GenerateFrameletFilter(frame);
W  = @(x) FraDecMultiLevel(x,D,Level); % Frame decomposition
WT = @(x) FraRecMultiLevel(x,R,Level); % Frame reconstruction
nD = length(D);

[m,n] = size(f1);

% Set up the option fields
[mit_inn,mit_out,tol,beta,Gamma_ini] = setopts(opts);

muLevel = getwThresh(rho/beta,wLevel,Level,D);

normg = norm(g,'fro');

% Initialization
if isempty(Gamma_ini); v = zeros(m,n); else v = Gamma_ini; clear Gamma_ini; end
d = W(v); b = d;

%% Main Loop

for nstep = 1 : mit_out
    
    for innerstep = 1 : mit_inn
        
        % Solve v
        C = CoeffOper('-',d,b);
        v = WT(C) + (f1-f2)/beta;
        v = min(max(v,0),1);
        
        % Solve d
        C = W(v);
        Cpb = CoeffOper('+',C,b);
        d = CoeffOper('vs',Cpb,muLevel);
        
    end
    
    err = 0;
    for ki = 1 : Level
        for ji = 1 : nD-1
            for jj = 1 : nD-1
                if ((ji~=1)||(jj~=1))||(ki==Level)
                    deltab = C{ki}{ji,jj}-d{ki}{ji,jj};
                    err = err + norm(deltab,'fro')^2;
                    b{ki}{ji,jj} = b{ki}{ji,jj} + deltab;
                end
            end
        end
    end
    
    err = sqrt(err)/normg;
    
    stopc = (err <= tol);
    
    if stopc == 1
        disp(['Problem Step = ' num2str(nstep) '; error = ' num2str(err)]);
        break
    end
end
if stopc ~= 1
    disp(['Problem Step = ' num2str(nstep) '; error = ' num2str(err)]);
end
Gamma = v;
GammaC = 1 - Gamma;

%% Subfunctions
function [mit_inn,mit_out,tol,beta,Gamma_ini] = setopts(opts)

% define default option fields
mit_inn = 1;
mit_out = 200;
tol = 1.e-4;
beta = 5;
Gamma_ini = [];

% change to specified option fields if exist
if ~isempty(opts);
    if ~isa(opts,'struct'); error('SplitBregDebP : opts not a struct'); end
    if isfield(opts,'mit_inn'); mit_inn = opts.mit_inn; end
    if isfield(opts,'mit_out'); mit_out = opts.mit_out; end
    if isfield(opts,'tol'); tol = opts.tol; end
    if isfield(opts,'beta'); beta = opts.beta; end
    if isfield(opts,'Gamma_ini'); Gamma_ini = opts.Gamma_ini; end
end
return;

function [D,R] = GenerateFrameletFilter(frame)

if isempty(frame); frame = 1; end
if (frame~=0) && (frame~=1) && (frame~=3)
    error('Input variable frame must be 0, 1, or 3');
end
if frame==0          %Haar Wavelet
    D{1}=[0 1 1]/2;
    D{2}=[0 1 -1]/2;
%     D{3}='rr';
    D{3}='cc';
    R{1}=[1 1 0]/2;
    R{2}=[-1 1 0]/2;
%     R{3}='rr';
    R{3}='cc';
elseif frame==1      %Piecewise Linear Framelet
    D{1}=[1 2 1]/4;
    D{2}=[1 0 -1]/4*sqrt(2);
    D{3}=[-1 2 -1]/4;
%     D{4}='rrr';
    D{4}='ccc';
    R{1}=[1 2 1]/4;
    R{2}=[-1 0 1]/4*sqrt(2);
    R{3}=[-1 2 -1]/4;
%     R{4}='rrr';
    R{4}='ccc';
elseif frame==3      %Piecewise Cubic Framelet
    D{1}=[1 4 6 4 1]/16;
    D{2}=[1 2 0 -2 -1]/8;
    D{3}=[-1 0 2 0 -1]/16*sqrt(6);
    D{4}=[-1 2 0 -2 1]/8;
    D{5}=[1 -4 6 -4 1]/16;
    D{6}='ccccc';
%     D{6}='rrrrr';
    R{1}=[1 4 6 4 1]/16;
    R{2}=[-1 -2 0 2 1]/8;
    R{3}=[-1 0 2 0 -1]/16*sqrt(6);
    R{4}=[1 -2 0 2 -1]/8;
    R{5}=[1 -4 6 -4 1]/16;
    R{6}='ccccc';
%     R{6}='rrrrr';
end
return;